'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Element2 = require('./Element');

var _Element3 = _interopRequireDefault(_Element2);

var _lines = require('../utils/lines');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Token = function (_Element) {
    _inherits(Token, _Element);

    _createClass(Token, null, [{
        key: 'create',

        /**
         * Generic token constructor.
         *
         * @param {String} type
         * @param {*} value
         */
        value: function create(type, value) {
            return new Token(type, value, valueToSourceCode(type, value));
        }

        /**
         * Creates new token using babel/acorn parser token.
         *
         * @param {{type: String, value: String, sourceCode: String}} token
         */

    }, {
        key: 'createFromToken',
        value: function createFromToken(token) {
            return new Token(token.type, token.value, token.sourceCode);
        }

        /**
         * @param {String} type
         * @param {String} value
         * @param {String} _sourceCode private source code argument
         */

    }]);

    function Token(type, value, _sourceCode) {
        _classCallCheck(this, Token);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Token).call(this, type, []));

        if (arguments.length === 2) {
            _sourceCode = valueToSourceCode(type, value);
        }

        var isComment = false;
        var isWhitespace = false;
        var isCode = true;
        switch (type) {
            case 'CommentLine':
                isComment = true;
                isCode = false;
                break;
            case 'CommentBlock':
                isComment = true;
                isCode = false;
                break;
            case 'Whitespace':
                isWhitespace = true;
                isCode = false;
                break;
            case 'AppleInstrumentationDirective':
            case 'GritDirective':
            case 'Hashbang':
                isCode = false;
                break;
        }
        _this.value = value;
        _this._sourceCode = _sourceCode;
        _this._sourceCodeLength = _sourceCode.length;
        _this._sourceCodeLines = (0, _lines.getLines)(_sourceCode);
        _this.isToken = true;
        _this.isComment = isComment;
        _this.isWhitespace = isWhitespace;
        _this.isCode = isCode;
        _this.isNonCodeToken = !isCode;
        return _this;
    }

    _createClass(Token, [{
        key: 'getFirstToken',
        value: function getFirstToken() {
            return this;
        }
    }, {
        key: 'getLastToken',
        value: function getLastToken() {
            return this;
        }
    }, {
        key: 'getSourceCode',
        value: function getSourceCode() {
            return this._sourceCode;
        }
    }, {
        key: 'getSourceCodeLength',
        value: function getSourceCodeLength() {
            return this._sourceCodeLength;
        }
    }, {
        key: 'getSourceCodeLines',
        value: function getSourceCodeLines() {
            return this._sourceCodeLines;
        }
    }, {
        key: 'getNewlineCount',
        value: function getNewlineCount() {
            return this._sourceCodeLines.length - 1;
        }
    }, {
        key: 'getValueLineInfo',
        value: function getValueLineInfo() {
            return (0, _lines.getLineInfo)(this.value);
        }
    }, {
        key: '_setChildren',
        value: function _setChildren(newChildren) {
            if (newChildren.length > 0) {
                throw new Error('Token nodes cannot contain child nodes');
            }

            this.childElements = newChildren;
        }

        /**
         * Clones current Element structure.
         *
         * @returns {Element}
         */

    }, {
        key: 'cloneElement',
        value: function cloneElement() {
            return new Token(this.type, this.value, this._sourceCode);
        }
    }]);

    return Token;
}(_Element3.default);

exports.default = Token;


function valueToSourceCode(type, value) {
    switch (type) {
        case 'CommentLine':
            return '//' + value;

        case 'CommentBlock':
            return '/*' + value + '*/';

        case 'RegularExpression':
            return String(value);

        case 'Numeric':
        case 'Boolean':
        case 'Null':
            return String(value);

        default:
            return value;
    }
}
//# sourceMappingURL=Token.js.map